==> graphic-0.4.0.tar.gz: lib/src/guide/axis/circular.dart <==
import 'dart:ui';

import 'package:flutter/painting.dart';
import 'package:graphic/src/common/label.dart';
import 'package:graphic/src/common/styles.dart';
import 'package:graphic/src/coord/polar.dart';
import 'package:graphic/src/graffiti/figure.dart';
import 'package:graphic/src/util/math.dart';
import 'package:graphic/src/util/path.dart';

import 'axis.dart';

List<Figure>? renderCircularAxis(
  List<TickInfo> ticks,
  double position,
  bool flip,
  StrokeStyle? line,
  PolarCoordConv coord,
) {
  final rst = <Figure>[];

  final flipSign = flip ? -1.0 : 1.0;
  final r = coord.startRadius + (coord.endRadius - coord.startRadius) * position;

  if (line != null) {
    rst.add(PathFigure(
      Path()..addArc(
        Rect.fromCircle(center: coord.center, radius: r),
        coord.startAngle,
        coord.endAngle - coord.startAngle,
      ),
      line.toPaint()
        ..style = PaintingStyle.stroke,
    ));
  }

  for (var tick in ticks) {
    // Polar coord dose not has tickLine.
    assert(tick.tickLine == null);

    final angle = coord.convertAngle(tick.position);
    if (angle >= coord.startAngle && angle <= coord.endAngle) {
      if (tick.label != null) {
        final labelAnchor = coord.polarToOffset(angle, r);
        Alignment defaultAlign;
        // According to anchor's quadrant.
        final anchorOffset = labelAnchor - coord.center;
        defaultAlign = Alignment(
          anchorOffset.dx.equalTo(0)
            ? 0
            : anchorOffset.dx / anchorOffset.dx.abs(),
          anchorOffset.dy.equalTo(0)
            ? 0
            : anchorOffset.dy / anchorOffset.dy.abs(),
        ) * flipSign;
        rst.add(renderLabel(
          Label(tick.text, tick.label!),
          labelAnchor,
          defaultAlign,
        ));
      }
    }
  }

  return rst.isEmpty ? null : rst;
}

List<Figure>? renderCircularGrid(
  List<TickInfo> ticks,
  PolarCoordConv coord,
) {
  final rst = <Figure>[];

  for (var tick in ticks) {
    if (tick.grid != null) {
      final angle = coord.convertAngle(tick.position);
      if (angle >= coord.startAngle && angle <= coord.endAngle) {
        rst.add(PathFigure(
          Paths.line(
            from: coord.polarToOffset(angle, coord.startRadius),
            to: coord.polarToOffset(angle, coord.endRadius),
          ),
          tick.grid!.toPaint(),
        ));
      }
    }
  }

  return rst.isEmpty ? null : rst;
}

==> graphic-0.4.1.tar.gz: lib/src/guide/axis/circular.dart <==
import 'dart:ui';

import 'package:flutter/painting.dart';
import 'package:graphic/src/common/label.dart';
import 'package:graphic/src/common/styles.dart';
import 'package:graphic/src/coord/polar.dart';
import 'package:graphic/src/graffiti/figure.dart';
import 'package:graphic/src/util/math.dart';
import 'package:graphic/src/util/path.dart';

import 'axis.dart';

List<Figure>? renderCircularAxis(
  List<TickInfo> ticks,
  double position,
  bool flip,
  StrokeStyle? line,
  PolarCoordConv coord,
) {
  final rst = <Figure>[];

  final flipSign = flip ? -1.0 : 1.0;
  final r =
      coord.startRadius + (coord.endRadius - coord.startRadius) * position;

  if (line != null) {
    rst.add(PathFigure(
      Path()
        ..addArc(
          Rect.fromCircle(center: coord.center, radius: r),
          coord.startAngle,
          coord.endAngle - coord.startAngle,
        ),
      line.toPaint()..style = PaintingStyle.stroke,
    ));
  }

  for (var tick in ticks) {
    // Polar coord dose not has tickLine.
    assert(tick.tickLine == null);

    final angle = coord.convertAngle(tick.position);
    if (angle >= coord.startAngle && angle <= coord.endAngle) {
      if (tick.label != null) {
        final labelAnchor = coord.polarToOffset(angle, r);
        Alignment defaultAlign;
        // According to anchor's quadrant.
        final anchorOffset = labelAnchor - coord.center;
        defaultAlign = Alignment(
              anchorOffset.dx.equalTo(0)
                  ? 0
                  : anchorOffset.dx / anchorOffset.dx.abs(),
              anchorOffset.dy.equalTo(0)
                  ? 0
                  : anchorOffset.dy / anchorOffset.dy.abs(),
            ) *
            flipSign;
        rst.add(renderLabel(
          Label(tick.text, tick.label!),
          labelAnchor,
          defaultAlign,
        ));
      }
    }
  }

  return rst.isEmpty ? null : rst;
}

List<Figure>? renderCircularGrid(
  List<TickInfo> ticks,
  PolarCoordConv coord,
) {
  final rst = <Figure>[];

  for (var tick in ticks) {
    if (tick.grid != null) {
      final angle = coord.convertAngle(tick.position);
      if (angle >= coord.startAngle && angle <= coord.endAngle) {
        rst.add(PathFigure(
          Paths.line(
            from: coord.polarToOffset(angle, coord.startRadius),
            to: coord.polarToOffset(angle, coord.endRadius),
          ),
          tick.grid!.toPaint(),
        ));
      }
    }
  }

  return rst.isEmpty ? null : rst;
}
